@@ -1409,6 +1409,18 @@ func TestFlipRange(t *testing.T) {
 		t.Error("Unexpected value: ", d.length)
 		return
 	}
+	//
+	for i := uint(0); i < 256; i++ {
+		for j := uint(0); j <= i; j++ {
+			bits := New(i)
+			bits.FlipRange(0, j)
+			c := bits.Count()
+			if c != j {
+				t.Error("Unexpected value: ", c, " expected: ", j)
+				return
+			}
+		}
+	}
 }
 
 func TestCopy(t *testing.T) {
@@ -1425,6 +1437,22 @@ func TestCopy(t *testing.T) {
 	}
 }
 
+func TestCopyFull(t *testing.T) {
+	a := New(10)
+	b := &BitSet{}
+	a.CopyFull(b)
+	if b.length != a.length || len(b.set) != len(a.set) {
+		t.Error("Expected full length copy")
+		return
+	}
+	for i, v := range a.set {
+		if v != b.set[i] {
+			t.Error("Unexpected value")
+			return
+		}
+	}
+}
+
 func TestNextSetError(t *testing.T) {
 	b := new(BitSet)
 	c, d := b.NextSet(1)
